Directory {
  dock: left;
  display: none;
  offset-x: -100%;
  max-width: 40;
  height: 100%;
  border: tall $primary-darken-1;
}

Directory._visible {
  offset-x: 0;
  display: block;
}

Stage {
  min-width: 30;
  max-width: 40;
  height: 100%;
  border: panel $primary;
  color: $secondary;
  border-title-align: center;

  border-title-color: black;
  border-title-background: white;
  border-title-style: bold;
}

Stage._actual {
  border: panel $secondary;
  color: $primary;
}

StagesContainer {
  height: 100%;
}

Task {
  width: 100%;
  height: auto;
  border: tall $success;
  margin: 1;
  padding: 1;
  color: $background-darken-3; /* ya no mover */
  background: $success-darken-1;
}

/* DONE: Cambiar colores de tareas seleccionadas */
/* DONE: Cambiar colores de la tarea que está seleccionada actualmente*/
._actual > Task {
  color: $background-darken-3;
  border: tall $warning;
  background: $warning-lighten-1;
}

._actual > Task._active {
  color: $background-darken-3;
  border: inner $error-lighten-2;
  background: $warning-lighten-1;
}

DeleteTask, AddTask {
  align: center middle;
}

#dialog {
  grid-size: 2;
  grid-gutter: 1 2;
  grid-rows: 1fr 3;
  padding: 0 1;
  width: 60;
  height: 11;
  border: thick $error-lighten-2 80%;
  background: $surface;
}

#question {
  column-span: 2;
  height: 1fr;
  width: 1fr;
  content-align: center middle;
}

Button {
  width: 100%;
}
